/*
 *  Copyright 2020-2023 Google LLC
 *  Copyright 2020-2023 EPAM Systems, Inc
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */

package org.opengroup.osdu.notification.provider.gcp.config;

import com.fasterxml.jackson.databind.ObjectMapper;
import java.util.Objects;
import lombok.RequiredArgsConstructor;
import org.opengroup.osdu.core.common.cache.ICache;
import org.opengroup.osdu.core.common.cache.VmCache;
import org.opengroup.osdu.core.common.model.notification.Subscription;
import org.opengroup.osdu.core.common.partition.PartitionInfo;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.data.redis.connection.RedisConfiguration;
import org.springframework.data.redis.connection.RedisConnectionFactory;
import org.springframework.data.redis.connection.RedisStandaloneConfiguration;
import org.springframework.data.redis.connection.lettuce.LettuceClientConfiguration;
import org.springframework.data.redis.connection.lettuce.LettuceClientConfiguration.LettuceClientConfigurationBuilder;
import org.springframework.data.redis.connection.lettuce.LettuceConnectionFactory;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.data.redis.serializer.Jackson2JsonRedisSerializer;
import org.springframework.data.redis.serializer.StringRedisSerializer;

@Configuration
@RequiredArgsConstructor
public class CacheConfig {

  @Bean
  public RedisTemplate<String, Subscription> setsTemplate(RedisConnectionFactory redisConnectionFactory) {
    RedisTemplate<String, Subscription> template = new RedisTemplate<>();
    Jackson2JsonRedisSerializer<Subscription> jsonRedisSerializer = new Jackson2JsonRedisSerializer<>(
        Subscription.class);
    ObjectMapper mapper = new ObjectMapper();
    jsonRedisSerializer.setObjectMapper(mapper);
    template.setValueSerializer(jsonRedisSerializer);
    template.setKeySerializer(new StringRedisSerializer());
    template.setHashKeySerializer(new StringRedisSerializer());
    template.setConnectionFactory(redisConnectionFactory);
    return template;
  }

  @Bean
  public RedisConnectionFactory redisConnectionFactory(RedisConfiguration redisConfiguration,
      RedisProperties redisProperties) {
    LettuceClientConfigurationBuilder configurationBuilder = LettuceClientConfiguration.builder();

    if (Boolean.TRUE.equals(redisProperties.getRedisWithSsl())) {
      configurationBuilder.useSsl();
    }
    configurationBuilder.commandTimeout(redisProperties.getTimeOut());
    LettuceClientConfiguration lettuceClientConfiguration = configurationBuilder.build();

    return new LettuceConnectionFactory(
        redisConfiguration,
        lettuceClientConfiguration
    );
  }

  @Bean
  public RedisConfiguration redisConfiguration(RedisProperties redisProperties) {
    RedisStandaloneConfiguration redisStandaloneConfiguration = new RedisStandaloneConfiguration();
    redisStandaloneConfiguration.setHostName(redisProperties.getRedisHost());
    redisStandaloneConfiguration.setPort(redisProperties.getRedisPort());
    if (Objects.nonNull(redisProperties.getRedisPassword()) && !redisProperties.getRedisPassword().isEmpty()) {
      redisStandaloneConfiguration.setPassword(redisProperties.getRedisPassword());
    }
    return redisStandaloneConfiguration;
  }

  @Bean
  public ICache<String, PartitionInfo> partitionInfoCache() {
    return new VmCache<>(600, 2000);
  }
}