/*
 *  Copyright 2020-2023 Google LLC
 *  Copyright 2020-2023 EPAM Systems, Inc
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */

package org.opengroup.osdu.notification.provider.gcp.pubsub.receiver;

import static org.opengroup.osdu.notification.provider.gcp.pubsub.OqmSubscriberManager.PUBLISH_SUFFIX;

import java.util.HashMap;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;
import lombok.extern.slf4j.Slf4j;
import org.opengroup.osdu.core.common.model.notification.Subscription;
import org.opengroup.osdu.core.gcp.oqm.driver.OqmDriver;
import org.opengroup.osdu.core.gcp.oqm.model.OqmAckReplier;
import org.opengroup.osdu.core.gcp.oqm.model.OqmDestination;
import org.opengroup.osdu.core.gcp.oqm.model.OqmMessage;
import org.opengroup.osdu.core.gcp.oqm.model.OqmMessageReceiver;
import org.opengroup.osdu.core.gcp.oqm.model.OqmSubscription;
import org.opengroup.osdu.core.gcp.oqm.model.OqmTopic;
import org.opengroup.osdu.notification.provider.gcp.service.ExternalSubscriptionsManager;
import org.opengroup.osdu.notification.provider.gcp.thread.ThreadScopeContextHolder;

/**
 * Used for event replication among existing subscribers in the Register service.
 */
@Slf4j
public class OqmEventReplicator implements OqmMessageReceiver {

    private final OqmSubscription subscription;
    private final OqmDriver driver;
    private final ExternalSubscriptionsManager externalSubscriptionsManager;

    public OqmEventReplicator(OqmSubscription subscription,
                                   OqmDriver driver,
        ExternalSubscriptionsManager externalSubscriptionsManager) {
        this.subscription = subscription;
        this.driver = driver;
        this.externalSubscriptionsManager = externalSubscriptionsManager;
    }

    @Override
    public void receiveMessage(OqmMessage message, OqmAckReplier replier) {
        String serviceTopic = subscription.getTopics().get(0).getName();
        log.debug("OQM | `{}` topic | Received message id: `{}`. Attributes: `{}`.", serviceTopic, message.getId(), message.getAttributes());

        try {
            handleMessage(message, serviceTopic);
        } catch (Exception e) {
            log.error("OQM | `{}` topic | Message not acknowledged by client", serviceTopic, e);
            replier.nack();
        } finally {
            ThreadScopeContextHolder.currentThreadScopeAttributes().clear();
        }

        log.debug("OQM | `{}` topic | Message acknowledged by client", serviceTopic);
        replier.ack();
    }

    private void handleMessage(OqmMessage message, String serviceTopic) {
        String dataPartitionId = message.getAttributes().get("data-partition-id");
        OqmTopic publishTopic = OqmTopic.builder()
                .name(serviceTopic + PUBLISH_SUFFIX)
                .build();

        Set<Subscription> cachedSubscriptionInfos = externalSubscriptionsManager.getSubscriptionsForTopic(
            dataPartitionId, serviceTopic);
        for (Subscription info : cachedSubscriptionInfos) {
            Map<String, String> attributes = new HashMap<>(refineAttributes(message));
            attributes.put("subscription-id", info.getNotificationId());
            attributes.put("topic", serviceTopic);
            OqmMessage enrichedMessage = OqmMessage.builder()
                    .id(message.getId())
                    .data(message.getData())
                    .attributes(attributes)
                    .build();
            driver.publish(enrichedMessage, publishTopic, OqmDestination.builder()
                    .partitionId(dataPartitionId)
                    .build());
        }
        log.debug("OQM | `{}` topic | Published notifications for `{}` third-party subscribers for tenant: `{}`",
                serviceTopic, cachedSubscriptionInfos.size(), dataPartitionId);
    }

    private static Map<String, String> refineAttributes(OqmMessage message) {
        Map<String, String> refinedAttributes = message.getAttributes().entrySet().stream()
                .filter(entry -> !entry.getKey().startsWith("goog"))
                .collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
        log.debug("Message attributes for message id: `{}` were refined: `{}`", message.getId(), refinedAttributes);
        return refinedAttributes;
    }
}